@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21337 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2008-10-15 03:52:04 +0000 (Wed, 15 Oct 2008) $
  */
 
 #ifdef __GNUC__
@@ -865,12 +865,12 @@ PBoolean PVXMLSession::Open(const PString & _mediaFormat)
   return Execute();
 }
 
+
 PBoolean PVXMLSession::Execute()
 {
   PWaitAndSignal m(sessionMutex);
 
-  // cannot open if no data is loaded
-  if (loaded && vxmlThread == NULL) {
+  if (vxmlThread == NULL) {
     threadRunning = PTrue;
     vxmlThread = PThread::Create(PCREATE_NOTIFIER(VXMLExecute), "VXML");
   }
@@ -909,9 +909,10 @@ PBoolean PVXMLSession::Close()
 void PVXMLSession::VXMLExecute(PThread &, INT)
 {
   while (!forceEnd && threadRunning) {
-
-    // process current node in the VXML script
-    ExecuteDialog();
+      
+      // process current node in the VXML script
+      if (currentNode != NULL)            
+        ExecuteDialog();
 
     // wait for something to happen
     if (currentNode == NULL || IsPlaying())
@@ -936,6 +937,7 @@ void PVXMLSession::VXMLExecute(PThread &, INT)
   return;
 }
 
+
 void PVXMLSession::ProcessUserInput()
 {
   // without this initialisation, gcc 4.1 gives a warning
@@ -1249,7 +1251,6 @@ PBoolean PVXMLSession::TraverseRecord()
     
     // create a semaphore, and then wait for the recording to terminate
     StartRecording(file, dtmfTerm, maxTime, termTime);
-    recordSync.Wait(maxTime);
     
     if (!recordSync.Wait(maxTime)) {
       // The Wait() has timed out, to signal that the record timed out.
@@ -1511,30 +1512,23 @@ PBoolean PVXMLSession::IsPlaying() const
   return (vxmlChannel != NULL) && vxmlChannel->IsPlaying();
 }
 
-PBoolean PVXMLSession::StartRecording(const PFilePath & /*_recordFn*/, 
-                                               PBoolean /*_recordDTMFTerm*/, 
-                              const PTimeInterval & /*_recordMaxTime*/, 
-                              const PTimeInterval & /*_recordFinalSilence*/)
+PBoolean PVXMLSession::StartRecording(const PFilePath & p_recordFn, 
+                                               PBoolean p_recordDTMFTerm, 
+                                  const PTimeInterval & p_recordMaxTime, 
+                                  const PTimeInterval & p_recordFinalSilence)
 {
-  /*
-  recording          = PTrue;
-  recordFn           = _recordFn;
-  recordDTMFTerm     = _recordDTMFTerm;
-  recordMaxTime      = _recordMaxTime;
-  recordFinalSilence = _recordFinalSilence;
+  recording          = true;
+  recordFn           = p_recordFn;
+  recordDTMFTerm     = p_recordDTMFTerm;
+  recordMaxTime      = p_recordMaxTime;
+  recordFinalSilence = p_recordFinalSilence;
 
-  if (incomingChannel != NULL) {
-    PXMLElement* element = (PXMLElement*) currentNode;
-    if ( element->HasAttribute("name")) {
-      PString chanName = element->GetAttribute("name");
-      incomingChannel->SetName(chanName);
-    }
-    return incomingChannel->StartRecording(recordFn, (unsigned )recordFinalSilence.GetMilliSeconds());
-  }
+  if (vxmlChannel != NULL) 
+    return vxmlChannel->StartRecording(recordFn,
+                                       (unsigned)recordFinalSilence.GetMilliSeconds(), 
+                                       (unsigned)recordMaxTime.GetMilliSeconds());
 
-  */
-
-  return PFalse;
+  return false;
 }
 
 void PVXMLSession::RecordEnd()
@@ -2456,18 +2450,16 @@ PBoolean PVXMLChannel::Read(void * buffer, PINDEX amount)
         }
 
         PTRACE(3, "PVXML\tFinished playing " << totalData << " bytes");
-        PDelayChannel::Close();
 
         // if current item still active, check for trailing actions
         if (currentPlayItem != NULL) {
           if (currentPlayItem->GetRepeat() > 1) {
-            if (!currentPlayItem->Rewind(GetBaseReadChannel())) {
-              PTRACE(3, "PVXML\tCannot rewind item - cancelling repeat");
-            } else {
+            if (currentPlayItem->Rewind(GetBaseReadChannel())) {
               currentPlayItem->SetRepeat(currentPlayItem->GetRepeat()-1);
               currentPlayItem->OnRepeat(*this);
               continue;
             }
+            PTRACE(3, "PVXML\tCannot rewind item - cancelling repeat");
           } 
 
           // see if end of queue delay specified
@@ -2487,6 +2479,8 @@ PBoolean PVXMLChannel::Read(void * buffer, PINDEX amount)
           delete currentPlayItem;
           currentPlayItem = NULL;
         }
+
+        PDelayChannel::Close();
       }
 
       // check the queue for the next action
@@ -2583,6 +2577,8 @@ PBoolean PVXMLChannel::QueueData(const PBYTEArray & PTRACE_PARAM(data), PINDEX r
     return PFalse;
   }
 
+  item->SetData(data);
+
   if (QueuePlayable(item))
     return PTrue;
 
@@ -2672,7 +2668,7 @@ static short beepData[] = { 0, 18784, 30432, 30400, 18784, 0, -18784, -30432, -3
 void PVXMLChannelPCM::GetBeepData(PBYTEArray & data, unsigned ms)
 {
   data.SetSize(0);
-  while (data.GetSize() < (PINDEX)((ms * 8) / 2)) {
+  while (data.GetSize() < (PINDEX)(ms * 16)) {
     PINDEX len = data.GetSize();
     data.SetSize(len + sizeof(beepData));
     memcpy(len + data.GetPointer(), beepData, sizeof(beepData));
